/*  Split Maze
 *  version 1, 2014
 *
 *  Copyright (C) 2014 Jari Komppa
 *
 *  This software is provided 'as-is', without any express or implied
 *  warranty.  In no event will the authors be held liable for any damages
 *  arising from the use of this software.
 *
 *  Permission is granted to anyone to use this software for any purpose,
 *  including commercial applications, and to alter it and redistribute it
 *  freely, subject to the following restrictions:
 *
 *  1. The origin of this software must not be misrepresented; you must not
 *     claim that you wrote the original software. If you use this software
 *     in a product, an acknowledgment in the product documentation would be
 *     appreciated but is not required.
 *  2. Altered source versions must be plainly marked as such, and must not be
 *     misrepresented as being the original software.
 *  3. This notice may not be removed or altered from any source distribution.
 *
 * Jari Komppa http://iki.fi/sol/
 *
 *************************************
 * 
 */

#include "toolkit.h"
#include "soloud.h"
#include "soloud_sfxr.h"
#include "soloud_wav.h"
#include "soloud_wavstream.h"
#include "soloud_biquadresonantfilter.h"

SoLoud::Soloud gSoloud;
//SoLoud::Sfxr gSndCrumb, gSndNextLevel;
SoLoud::Wav gSndCollided, gSndCrumb, gSndNextLevel;
SoLoud::WavStream gSndMusicTimed, gSndMusicEndless, gSndMusicMenu;
SoLoud::Bus gSndFxBus, gMusicBus;
SoLoud::BiquadResonantFilter gMusicFilter;

int fxhandle, musichandle, musichandletimed, musichandleendless, musichandlemenu;

enum STATES
{
	STATE_MENU,
	STATE_INGAME,
	STATE_PAUSE,
	STATE_GAMEOVER
};

enum GAMETYPE
{
	GAME_ENDLESS,
	GAME_TIMED
};


int gStateStartTick = 0;
int gState = STATE_MENU;
int gGameType = GAME_ENDLESS;
int gTimeleft = 10 * 1000;

int timelefts[1000];

int mapwidth = 10;
int mapheight = 15;

char *gMap[2] = {0, 0};
char *gWholeMap = 0;
char *gColliders = 0;
char *gBevels[2] = {0, 0};


float gCamx = 5;
float gCamy = 5;

float playerx = 5;
float playery = 5;

float playermx = 0;
float playermy = 0;

float destx = 0;
float desty = 0;

int gLevel = 0;
float gRoll = 0;
int gPathLen = 0;

// Pointer to the joystick interface
SDL_Joystick *gJoystick = NULL;

#define TITLE "Split Maze 1.0"

// How big a tick difference is considered 'time warp', i.e. skip the time
// (to avoid physics blowing up)
#define TICK_TIMEWARP 1000
// Physics ticks per second
#define PHYSICS_FPS 100

#define MAXVTX 32

//#define USE_PERFCOUNTERS

enum BEVELS
{
	B_UP = 1,
	B_DOWN = 2,
	B_LEFT = 4,
	B_RIGHT = 8
};

enum colliderenum
{
  COLLIDE_N  = 0x01,
  COLLIDE_NW = 0x02,
  COLLIDE_W  = 0x04,
  COLLIDE_SW = 0x08,
  COLLIDE_S  = 0x10,
  COLLIDE_SE = 0x20,
  COLLIDE_E  = 0x40,
  COLLIDE_NE = 0x80
};

int tex_tile, tex_font, tex_wall;
int gKeyState = 0;
int gLastTick = 0;


void setstate(int nextstate)
{
	int i;
	switch (nextstate)
	{
	case STATE_PAUSE:
		gSoloud.fadeFilterParameter(musichandle,0,0,1,0.5);
		break;
	case STATE_INGAME:
		gSoloud.fadeFilterParameter(musichandle,0,0,0,0.5);
		gSoloud.fadeVolume(musichandlemenu, 0, 1);
		gSoloud.schedulePause(musichandlemenu, 1);
		if (gGameType == GAME_TIMED)
			i = musichandletimed;
		else
			i = musichandleendless;
		gSoloud.setPause(i, 0);
		gSoloud.fadeVolume(i, 1, 1);
		break;
	case STATE_GAMEOVER:
		gSoloud.setPause(musichandlemenu, 0);
		gSoloud.fadeVolume(musichandlemenu, 1, 4);
		if (gGameType == GAME_TIMED)
			i = musichandletimed;
		else
			i = musichandleendless;
		gSoloud.fadeVolume(i, 0, 1);
		gSoloud.schedulePause(i, 1);
		break;
	case STATE_MENU:
		gSoloud.setPause(musichandlemenu, 0);
		gSoloud.fadeVolume(musichandlemenu, 1, 1);
		if (gGameType == GAME_TIMED)
			i = musichandletimed;
		else
			i = musichandleendless;
		gSoloud.fadeVolume(i, 0, 1);
		gSoloud.schedulePause(i, 1);
		break;
	}
	gState = nextstate;
}

void handle_key(int keysym, int down)
{
    switch(keysym)
    {
    case SDLK_ESCAPE:
        if (!down)
        {
			if (gState == STATE_MENU)
				exit(0);
			if (gState == STATE_INGAME)
				setstate(STATE_PAUSE);
			else
				setstate(STATE_MENU);
        }
        break;
    case SDLK_UP:
    case SDLK_w:
        if (down) gKeyState |= KEY_UP; else gKeyState &= ~KEY_UP;
        break;
    case SDLK_DOWN:
    case SDLK_s:
        if (down) gKeyState |= KEY_DOWN; else gKeyState &= ~KEY_DOWN;
        break;
    case SDLK_LEFT:
    case SDLK_a:
        if (down) gKeyState |= KEY_LEFT; else gKeyState &= ~KEY_LEFT;
        break;
    case SDLK_RIGHT:
    case SDLK_d:
        if (down) gKeyState |= KEY_RIGHT; else gKeyState &= ~KEY_RIGHT;
        break;
    case SDLK_RCTRL:
    case SDLK_LCTRL:
	case SDLK_RETURN:
	case SDLK_SPACE:
        if (down) gKeyState |= KEY_FIRE; else gKeyState &= ~KEY_FIRE;
        break;
    }
}

void process_events()
{
    SDL_Event event;

    while (SDL_PollEvent(&event)) 
    {
        switch (event.type) 
        {
        case SDL_KEYDOWN:
            handle_key(event.key.keysym.sym, 1);
			// If a key is pressed, report it to the widgets
			gUIState.keyentered = event.key.keysym.sym;
			gUIState.keymod = event.key.keysym.mod;
			// if key is ASCII, accept it as character input
			if ((event.key.keysym.unicode & 0xFF80) == 0)
				gUIState.keychar = event.key.keysym.unicode & 0x7f;				
            break;
        case SDL_KEYUP:
            handle_key(event.key.keysym.sym, 0);
            break;
        case SDL_MOUSEMOTION:
			// update mouse position
			gUIState.mousex = (event.motion.x * (float)DESIRED_WINDOW_WIDTH) / (float)gScreenWidth;
			gUIState.mousey = (event.motion.y * (float)DESIRED_WINDOW_HEIGHT) / (float)gScreenHeight;
            break;
		case SDL_MOUSEBUTTONDOWN:
			// update button down state if left-clicking
			if (event.button.button == 1)
            {
				gUIState.mousedown = 1;
			    gUIState.mousedownx = (event.motion.x * (float)DESIRED_WINDOW_WIDTH) / (float)gScreenWidth;
			    gUIState.mousedowny = (event.motion.y * (float)DESIRED_WINDOW_HEIGHT) / (float)gScreenHeight;
            }
            if (event.button.button == 4)
            {
                gUIState.scroll = +1;
            }
            if (event.button.button == 5)
            {
                gUIState.scroll = -1;
            }
			break;
		case SDL_MOUSEBUTTONUP:
			// update button down state if left-clicking
			if (event.button.button == 1)
				gUIState.mousedown = 0;
			break;
        case SDL_QUIT:
			if (gJoystick)
				SDL_JoystickClose(gJoystick);
            SDL_Quit();
            exit(0);
            break;
        case SDL_VIDEORESIZE:
            gScreenWidth = event.resize.w;
            gScreenHeight = event.resize.h;
            initvideo(0);
            break;

        }
    }
}

struct Crumb
{
	float x, y, live;
	int side;
};

#define MAXCRUMBS 100
Crumb gCrumb[MAXCRUMBS];

#define MAXBLINKS 20
#define BLINKAGE 20
Crumb gBlink[MAXBLINKS];

#define TILES_X 1
#define TILES_Y 8
#define NUDGE 32.0

struct vertex
{
  float x, y, z;
};

// Original vertices
vertex *gVtx;
// Transformed vertices
vertex *gRVtx;

void rotate_z(float angle)
{
  float ca = (float)cos(angle);
  float sa = (float)sin(angle);
  int i;
  for (i = 0; i < MAXVTX; i++)
  {
    float x = gRVtx[i].x * ca - gRVtx[i].y * sa;
    float y = gRVtx[i].x * sa + gRVtx[i].y * ca;
    gRVtx[i].x = x;
    gRVtx[i].y = y;
  }
}

void rotate_y(float angle)
{
  float ca = (float)cos(angle);
  float sa = (float)sin(angle);
  int i;
  for (i = 0; i < MAXVTX; i++)
  {
    float z = gRVtx[i].z * ca - gRVtx[i].x * sa;
    float x = gRVtx[i].z * sa + gRVtx[i].x * ca;
    gRVtx[i].z = z;
    gRVtx[i].x = x;
  }
}

void rotate_x(float angle)
{
  float ca = (float)cos(angle);
  float sa = (float)sin(angle);
  int i;
  for (i = 0; i < MAXVTX; i++)
  {
    float y = gRVtx[i].y * ca - gRVtx[i].z * sa;
    float z = gRVtx[i].y * sa + gRVtx[i].z * ca;
    gRVtx[i].y = y;
    gRVtx[i].z = z;
  }
}

void addblink(int aX, int aY)
{
	int i;
	int free = -1;
	for (i = 0; i < MAXBLINKS; i++)
	{
		if (gBlink[i].live == 0)
		{
			free = i;
		}
		else
		if (gBlink[i].x == aX && gBlink[i].y == aY)
		{
			gBlink[i].live = BLINKAGE;
			return;
		}
	}

	if (free >= 0)
	{
		gBlink[free].live = BLINKAGE;
		gBlink[free].x = aX;
		gBlink[free].y = aY;
		gBlink[free].side = 0;
		if (gMap[0][aY*mapwidth+aX] == '#')
			gBlink[free].side |= 1;
		if (gMap[1][aY*mapwidth+aX] == '#')
			gBlink[free].side |= 2;

		if (gBlink[free].side == 3)
			gSndFxBus.play(gSndCollided,1);
		else
		{
			if (gBlink[free].side & 1)
				gSndFxBus.play(gSndCollided,1,-0.5);
			if (gBlink[free].side & 2)
				gSndFxBus.play(gSndCollided,1,0.5);
		}
	}
}

void drawtile(int tex, float x, float y, float w, float h, int tile, int color = 0xffffffff)
{
	x *= 2;
	w *= 2;

	float ustep = 1.0 / TILES_X;
	float vstep = 1.0 / TILES_Y;

	float uofs = (tile / TILES_Y) * ustep;
	float vofs = (tile % TILES_Y) * vstep;

	uofs += 1 / (NUDGE * TILES_X);
	vofs += 1 / (NUDGE * TILES_Y);
	ustep -= 2 / (NUDGE * TILES_X);
	vstep -= 2 / (NUDGE * TILES_Y);

    glBindTexture(GL_TEXTURE_2D, tex);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	glColor4f(((color >> 16) & 0xff) / 256.0f,
              ((color >> 8) & 0xff) / 256.0f,
              ((color >> 0) & 0xff) / 256.0f,
              ((color >> 24) & 0xff) / 256.0f);

    glBegin(GL_TRIANGLE_STRIP);
      glTexCoord2f(uofs,vofs);
      glVertex2f(x - w / 2,y - h / 2);
      glTexCoord2f(uofs,vofs+vstep);
      glVertex2f(x - w / 2,y + h / 2);
      glTexCoord2f(uofs+ustep,vofs);
      glVertex2f(x + w / 2,y - h / 2);
      glTexCoord2f(uofs+ustep,vofs+vstep);
      glVertex2f(x + w / 2,y + h / 2);
    glEnd();
    glDisable(GL_TEXTURE_2D);
}

void drawwall(int tex, float x, float y, float w, float h, int tile, int color, int bevels)
{
	x *= 2;
	w *= 2;

	float ustep = 1.0 / 8;
	float vstep = 1.0 / 8;

	float uofs = (tile / 8) * ustep;
	float vofs = (tile % 8) * vstep;

	uofs += 1 / (NUDGE * 16);
	vofs += 1 / (NUDGE * 16);
	ustep -= 2 / (NUDGE * 16);
	vstep -= 2 / (NUDGE * 16);

    glBindTexture(GL_TEXTURE_2D, tex);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	glColor4f(((color >> 16) & 0xff) / 256.0f,
              ((color >> 8) & 0xff) / 256.0f,
              ((color >> 0) & 0xff) / 256.0f,
              ((color >> 24) & 0xff) / 256.0f);

    glBegin(GL_TRIANGLE_STRIP);
      glTexCoord2f(uofs,vofs);
      glVertex2f(x - w / 2,y - h / 2);
      glTexCoord2f(uofs,vofs+vstep);
      glVertex2f(x - w / 2,y + h / 2);
      glTexCoord2f(uofs+ustep,vofs);
      glVertex2f(x + w / 2,y - h / 2);
      glTexCoord2f(uofs+ustep,vofs+vstep);
      glVertex2f(x + w / 2,y + h / 2);
    glEnd();
    glDisable(GL_TEXTURE_2D);

#define BEVEL_WIDTH 6

	if (bevels & B_UP)
		drawrect(x-w/2, y-h/2, w, BEVEL_WIDTH, 0xafffffff);
	if (bevels & B_DOWN)
		drawrect(x-w/2, y+h-BEVEL_WIDTH-h/2, w, BEVEL_WIDTH, 0x7f000000);
	if (bevels & B_LEFT)
		drawrect(x-w/2, y-h/2, BEVEL_WIDTH*2, h, 0xafffffff);
	if (bevels & B_RIGHT)
		drawrect(x-w/2+w-BEVEL_WIDTH*2, y-h/2, BEVEL_WIDTH*2, h, 0x7f000000);
}


void drawcircle(float aX, float aY, float aRx, float aRy, int color)
{
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	glColor4f(((color >> 16) & 0xff) / 256.0f,
              ((color >> 8) & 0xff) / 256.0f,
              ((color >> 0) & 0xff) / 256.0f,
              ((color >> 24) & 0xff) / 256.0f);

	int d = floor(max(aRx, aRy));
	int i;
	glBegin(GL_TRIANGLE_FAN);
		for (i = 0; i < d; i++)
		glVertex2f(sin(2 * M_PI * i / (float)d) * aRx + aX, cos(2 * M_PI * i / (float)d) * aRy + aY);
	glEnd();

}

void vidsplit(int split)
{
	if (split == 1)
	{
#ifdef DESIRED_ASPECT
		float aspect = DESIRED_ASPECT;
		if (((float)gScreenWidth / gScreenHeight) > aspect)
		{
			float realx = gScreenHeight * aspect;
			float extrax = gScreenWidth - realx;

			glViewport( extrax / 2, 0, realx/2 - 2, gScreenHeight );
		}
		else
		{
			float realy = gScreenWidth / aspect;
			float extray = gScreenHeight - realy;

			glViewport( 0, extray / 2, gScreenWidth/2 - 2, realy );
		}
#else
	    glViewport( 0, 0, gScreenWidth/2, gScreenHeight );
#endif
		return;
	}

	if (split == 2)
	{
#ifdef DESIRED_ASPECT
		float aspect = DESIRED_ASPECT;
		if (((float)gScreenWidth / gScreenHeight) > aspect)
		{
			float realx = gScreenHeight * aspect;
			float extrax = gScreenWidth - realx;

			glViewport( extrax / 2 + realx / 2 + 4, 0, realx/2 - 2, gScreenHeight );
		}
		else
		{
			float realy = gScreenWidth / aspect;
			float extray = gScreenHeight - realy;

			glViewport( gScreenWidth/2 + 4, extray / 2, gScreenWidth/2 - 2, realy );
		}
#else
	    glViewport(gScreenWidth/2, 0, gScreenWidth/2, gScreenHeight );
#endif
		return;
	}

#ifdef DESIRED_ASPECT
	float aspect = DESIRED_ASPECT;
	if (((float)gScreenWidth / gScreenHeight) > aspect)
	{
		float realx = gScreenHeight * aspect;
		float extrax = gScreenWidth - realx;

		glViewport( extrax / 2, 0, realx, gScreenHeight );
	}
	else
	{
		float realy = gScreenWidth / aspect;
		float extray = gScreenHeight - realy;

		glViewport( 0, extray / 2, gScreenWidth, realy );
	}
#else
	glViewport( 0, 0, gScreenWidth, gScreenHeight );
#endif
}

void drawsiderect(float x, float y, float w, float h, int c)
{
	if (x < 0) x = 0;
	if (y < 0) y = 0;
	if (x > 800-w) x = 800-w;
	if (y > 450-h) y = 450-h;
	drawrect(x,y,w,h,c);
}

void analyse_map()
{
	delete[] gColliders;
	delete[] gBevels[0];
	delete[] gBevels[1];
	gBevels[0] = new char[mapheight * mapwidth];
	gBevels[1] = new char[mapheight * mapwidth];
	gColliders = new char[mapheight * mapwidth];
	memset(gColliders,0,mapheight * mapwidth);
	memset(gBevels[0],0,mapwidth * mapheight);
	memset(gBevels[1],0,mapwidth * mapheight);
	int i, j, c;
	for (i = 0, c = 0; i < mapheight; i++)
	{
		for (j = 0; j < mapwidth; j++, c++)
		{
			int split;
			for (split = 0; split < 2; split++)
			{
				if (gMap[split][c] == 'p')
				{
					playerx = j;
					playery = i;
				}

				if (gMap[split][c] == 'n')
				{
					destx = j;
					desty = i;
				}

				if (gMap[split][c] == '#')
				{
					if (i == 0 || gMap[split][c - mapwidth] != '#')
						gBevels[split][c] |= B_UP;
					if (i == mapheight-1 || gMap[split][c + mapwidth] != '#')
						gBevels[split][c] |= B_DOWN;
					if (j == 0 || gMap[split][c - 1] != '#')
						gBevels[split][c] |= B_LEFT;
					if (j == mapwidth-1 || gMap[split][c + 1] != '#')
						gBevels[split][c] |= B_RIGHT;
				}
			}

			if (gWholeMap[c] == '#')
			{
				if (i > 0)
				{
					if (j > 0)
						gColliders[c - mapwidth - 1] |= COLLIDE_SE;

					gColliders[c - mapwidth] |= COLLIDE_S;

					if (j < mapwidth - 1)
						gColliders[c - mapwidth + 1] |= COLLIDE_SW;
				}

				if (j > 0)
					gColliders[c - 1] |= COLLIDE_E;

				if (j < mapwidth - 1)
					gColliders[c + 1] |= COLLIDE_W;

				if (i < mapheight - 1)
				{
					if (j > 0)
						gColliders[c + mapwidth - 1] |= COLLIDE_NE;

					gColliders[c + mapwidth] |= COLLIDE_N;

					if (j < mapwidth - 1)
						gColliders[c + mapwidth + 1] |= COLLIDE_NW;
				}
			}
		}
	}
	playermx = 0;
	playermy = 0;
	gCamx = playerx;
	gCamy = playery;
}

int solvable(char *aMap, int aSprinkle = 0)
{
	int s = mapwidth * mapheight;
	int *t = new int[s];
	memset(t, 0, sizeof(int) * s);
	int changed = 1;
	int i, j;

	for (i = 0; i < s; i++)
	{
		if (aMap[i] == 'p')
			t[i] = 1;
		if (aMap[i] == '#')
			t[i] = -1;
	}

	while (changed)
	{
		changed = 0;
		for (i = 1; i < mapwidth - 1; i++)
		{
			for (j = 1; j < mapheight - 1; j++)
			{
				int p = j * mapwidth + i;
				if (t[p] == 0)
				{
					if (t[p - 1] > 0 ||
						t[p + 1] > 0 ||
						t[p - mapwidth] > 0 ||
						t[p + mapwidth] > 0)
					{
						t[p] = max(t[p - 1], max(t[p + 1], max(t[p - mapwidth], t[p + mapwidth]))) + 1;
						changed = 1;

						if (aMap[p - 1] == 'n' ||
							aMap[p + 1] == 'n' ||
							aMap[p - mapwidth] == 'n' ||
							aMap[p + mapwidth] == 'n')
						{
							gPathLen = t[p];
							if (aSprinkle)
							{
								while (t[p] != 1)
								{
									if (aMap[p] == '.' && (gPhysicsRand.genrand_int31() % 4) == 0)
										aMap[p] = ',';
									if (t[p - 1] > 0 && t[p - 1] < t[p]) p -= 1; else
									if (t[p + 1] > 0 && t[p + 1] < t[p]) p += 1; else
									if (t[p - mapwidth] > 0 && t[p - mapwidth] < t[p]) p -= mapwidth; else
									if (t[p + mapwidth] > 0 && t[p + mapwidth] < t[p]) p += mapwidth; else
									{
										p = p;
									}
								}
							}

							delete[] t;
							return 1;
						}
					}
				}
			}
		}
	}

	delete[] t;
	return 0;
}

void showbusy()
{
    glClearColor(1,1,1,1.0);
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	int i;
	for (i = 0; i < 2; i++)
	{
		int r = gVisualRand.genrand_int31() % 50 + 10;
		drawcircle(gVisualRand.genrand_int31() % 800, gVisualRand.genrand_int31() % 450, r, r, 0xff000000);
	}

    SDL_Delay(10);
	SDL_GL_SwapBuffers();
}

void genmap(int aLevel)
{
	switch (gGameType)
	{
	case GAME_TIMED:
		gPhysicsRand.init_genrand(aLevel + 0x7aa71ee7);
		break;
	case GAME_ENDLESS:
		gPhysicsRand.init_genrand(SDL_GetTicks());
		break;
	}


	int maxw = aLevel * 3;
	int maxh = aLevel * 3;
	if (maxw > 92) maxw = 92;
	if (maxh > 92) maxh = 92;

	mapwidth = 8 + ((gPhysicsRand.genrand_int31() % 100) * maxw) / 100;
	mapheight = 8 + ((gPhysicsRand.genrand_int31() % 100) * maxh) / 100;
	int s = mapwidth * mapheight;
	gWholeMap = new char[s + 1];
	gMap[0] = new char[s + 1];
	gMap[1] = new char[s + 1];

	memset(gWholeMap, '.', s);

	int i;
	for (i = 0; i < mapwidth; i++)
	{
		gWholeMap[i] = '#';
		gWholeMap[i + s - mapwidth] = '#';
	}
	
	for (i = 0; i < mapheight; i++)
	{
		gWholeMap[i * mapwidth] = '#';
		gWholeMap[i * mapwidth + mapwidth - 1] = '#';
	}

	int x = 0, y = 0;
	int x1 = 0, y1 = 0, x2 = 0, y2 = 0;
	float d = 0;
	while (d < (mapwidth + mapheight) / 3)
	{
		x1 = (gPhysicsRand.genrand_int31() % (mapwidth - 2)) + 1;
		y1 = (gPhysicsRand.genrand_int31() % (mapheight - 2)) + 1;
		x2 = (gPhysicsRand.genrand_int31() % (mapwidth - 2)) + 1;
		y2 = (gPhysicsRand.genrand_int31() % (mapheight - 2)) + 1;
		d = sqrt((float)(x2-x1)*(x2-x1)+(y2-y1)*(y2-y1));
	}
	gWholeMap[y1 * mapwidth + x1] = 'p';
	gWholeMap[y2 * mapwidth + x2] = 'n';

	memcpy(gMap[0], gWholeMap, s);
	memcpy(gMap[1], gWholeMap, s);

	int iters = 0;

	int retries = 0;

	int oldtick = SDL_GetTicks();

	while (retries < 10)
	{
		while (iters < 2000 && solvable(gWholeMap))
		{
			do
			{
				x = gPhysicsRand.genrand_int31() % mapwidth;
				y = gPhysicsRand.genrand_int31() % mapheight;
			}
			while (gWholeMap[y * mapwidth + x] != '.');
			gWholeMap[y * mapwidth + x] = '#';
			iters++;

			int t = SDL_GetTicks();
			if (t - oldtick > 100)
			{
				showbusy();
				oldtick = t;
			}
		}
		gWholeMap[y * mapwidth + x] = '.';
		retries++;
	}

	solvable(gWholeMap, 1);

	for (i = 0; i < s; i++)
	{
		switch (gPhysicsRand.genrand_int31() % 5)
		{
		default:
		case 0:
			gMap[0][i] = gWholeMap[i];
			gMap[1][i] = gWholeMap[i];
			break;
		case 3:
		case 1:
			gMap[0][i] = gWholeMap[i];
			break;
		case 2:
		case 4:
			gMap[1][i] = gWholeMap[i];
			break;
		}
	}
}

int mapxlate[99] = 
{
	1, 78, 2, 4, 3, 5, 96, 6, 92, 93, 66, 19, 61, 91,
	7, 22, 30, 11, 82, 21, 23, 40, 36, 16, 52, 74, 27,
	75, 14, 73, 8, 95, 72, 9, 12, 99, 10, 65, 17, 43,
	48, 13, 26, 87, 90, 94, 71, 18, 41, 77, 28, 34, 20,
	69, 15, 60, 38, 62, 24, 88, 67, 57, 76, 37, 56, 47, 
	25, 39, 63, 89, 49, 80, 54, 70, 55, 59, 31, 51, 64, 
	86, 98, 58, 29, 68, 45, 35, 42, 53, 44, 81, 97, 79, 
	83, 46, 85, 84, 50, 32, 33
};


void loadmap(int aLevel)
{
	gSndFxBus.play(gSndNextLevel);

	if (aLevel < 99 && gGameType == GAME_TIMED)
	{
		aLevel = mapxlate[aLevel-1];
	}

	delete[] gMap[0];
	delete[] gMap[1];
	delete[] gWholeMap;

	switch (aLevel)
	{
		/*
	case 0:
	
		mapwidth = 20;
		mapheight = 15;
		gMap[0] = new char[mapwidth * mapheight + 1];
		gMap[1] = new char[mapwidth * mapheight + 1];
		memcpy(gMap[0], tutmap1, mapwidth * mapheight + 1);
		memcpy(gMap[1], tutmap2, mapwidth * mapheight + 1);
		break;*/
	default:
	case 1:
		genmap(aLevel);
		break;
	}
	analyse_map();

	int i;
	for (i = 0; i < MAXCRUMBS; i++)
		gCrumb[i].live = 0;

	gStateStartTick = SDL_GetTicks();
}

float rotz = 0, roty = 0;

void physics_tick(int tick)
{
	int i = 0;
	float my = 0;
	float mx = 0;

	if (gGameType == GAME_TIMED)
		gTimeleft -= 1000 / PHYSICS_FPS;
    
	if (gJoystick)
    {
      mx += (SDL_JoystickGetAxis(gJoystick, 0) / 32768.0f);
      my += (SDL_JoystickGetAxis(gJoystick, 1) / 32768.0f);
    }

	if (mx == 0 && my == 0)
	{
		if (gKeyState & KEY_UP)
			my -= 1;
		if (gKeyState & KEY_DOWN)
			my += 1;
		if (gKeyState & KEY_LEFT)
			mx -= 1;
		if (gKeyState & KEY_RIGHT)
			mx += 1;
	}

	float oldpx = playerx;
	float oldpy = playery;

	playermx += mx * 0.0061;
	playermy += my * 0.0061;
	playermx *= 0.93;
	playermy *= 0.93;
	playerx += playermx;
	playery += playermy;

	int tilex = (int)floor(playerx + 0.5);
	int tiley = (int)floor(playery + 0.5);
/*
	int tilexl = (int)floor(playerx + 0.4);
	int tilexr = (int)floor(playerx + 0.6);
	int tileyu = (int)floor(playery + 0.4);
	int tileyd = (int)floor(playery + 0.6);
*/
	float tx = playerx - floor(playerx + 0.5);
	float ty = playery - floor(playery + 0.5);

	int currenttile = tiley * mapwidth + tilex;
	int collided = 0;

	float normalx = 0;
	float normaly = 0;

	if (gColliders[currenttile] & COLLIDE_E && tx > 0.35) 
	{
		addblink(tilex + 1, tiley);
		normalx -= tx - 0.35;
		collided = 3;
	}
	
	if (gColliders[currenttile] & COLLIDE_W && tx < -0.35) 
	{
		addblink(tilex - 1, tiley);
		normalx -= tx + 0.35;
		collided = 3;
	}
	
	if (gColliders[currenttile] & COLLIDE_N && ty < -0.35) 
	{
		addblink(tilex, tiley - 1);
		normaly -= ty + 0.35;
		collided = 3;
	}

	if (gColliders[currenttile] & COLLIDE_S && ty > 0.35) 	
	{
		addblink(tilex, tiley + 1);
		normaly -= ty - 0.35;
		collided = 3;
	}

	if (collided)
	{
		tx = (playerx + normalx) - floor(playerx + normalx + 0.5);
		ty = (playery + normaly) - floor(playery + normaly + 0.5);
	}
	
	if (gColliders[currenttile] & COLLIDE_NE && tx > 0.4 && ty < -0.4)
	{
		addblink(tilex + 1, tiley - 1);
		normaly -= ty + 0.4;
		normalx -= tx - 0.4;
		collided = 3;
	}

	if (gColliders[currenttile] & COLLIDE_SE && tx > 0.4 && ty > 0.4)
	{
		addblink(tilex + 1, tiley + 1);
		normaly -= ty - 0.4;
		normalx -= tx - 0.4;
		collided = 3;
	}

	if (gColliders[currenttile] & COLLIDE_SW && tx < -0.4 && ty > 0.4)
	{
		addblink(tilex - 1, tiley + 1);
		normaly -= ty - 0.4;
		normalx -= tx + 0.4;
		collided = 3;
	}

	if (gColliders[currenttile] & COLLIDE_NW && tx < -0.4 && ty < -0.4)
	{
		addblink(tilex - 1, tiley - 1);
		normaly -= ty + 0.4;
		normalx -= tx + 0.4;
		collided = 3;
	}
	
	/*

	char temp[1024];
	sprintf(temp, "%3.3f %3.3f %d %d %X", tx, ty, tilex, tiley, gColliders[currenttile]);
	SDL_WM_SetCaption(temp, NULL);
	*/


	if (gWholeMap[currenttile] == 'n') collided = 20;
	if (gWholeMap[currenttile] == ',') collided = 30;

	if (collided == 30)
	{
		// pick up crumb
		gMap[0][tiley * mapwidth + tilex] = '.';
		gMap[1][tiley * mapwidth + tilex] = '.';
		gWholeMap[tiley * mapwidth + tilex] = '.';
		gSndFxBus.play(gSndCrumb);
		return;
	}

	if (collided == 20)
	{
		// finished
		gTimeleft += 1000 * gPathLen / 4;
		timelefts[gLevel-1] = gTimeleft;
		gLevel++;
		loadmap(gLevel);
		return;
	}

	if (collided)
	{
		playerx += normalx;
		playery += normaly;

		float l = sqrt(normalx*normalx + normaly*normaly);
		if (l != 0)
		{
			normalx /= l;
			normaly /= l;
		}

		float dot = playermx * normalx + playermy * normaly;

		playermx -= dot * 1.5 * normalx;
		playermy -= dot * 1.5 * normaly;
	}

	gRoll += (float)sqrt(playermx * playermx + playermy * playermy);

	rotz = atan2(playermy, playermx);

	static int periodical = 0;

	if (abs(playermx)+abs(playermy) > 0.05)
	{
		periodical++;
		if (periodical >= 2)
		{
			periodical = 0;

			i = gPhysicsRand.genrand_int31() % MAXCRUMBS;
			if (gCrumb[i].live == 0)
			{
				gCrumb[i].x = playerx;
				gCrumb[i].y = playery;
				gCrumb[i].live = gPhysicsRand.genrand_int31() % 400 + 600;
			}
		}
	}
	for (i = 0; i < MAXCRUMBS; i++)
		if (gCrumb[i].live)
			gCrumb[i].live--;

	for (i = 0; i < MAXBLINKS; i++)
	{
		if (gBlink[i].live)
		{
			gBlink[i].live--;
		}
	}

	float prex = playerx + playermx * 20;
	float prey = playery + playermy * 20;

	gCamx = (gCamx * 15 + prex) / 16;
	gCamy = (gCamy * 15 + prey) / 16;
}

void pause_tick(int tick)
{
	int fire = !!(gKeyState & KEY_FIRE);

	if (gJoystick)
	{
		int i;
		for (i = 0; i < 16; i++)
			if (SDL_JoystickGetButton(gJoystick, i))
				fire = 1;
	}

	if (fire)
	{
		setstate(STATE_INGAME);
	}
}

float bounce(int tick)
{
	return abs(sin(tick * 0.00423));
}

void drawplayer(float aX, float aY, int side)
{
	drawcircle((aX+2)*2,aY+4+6,32,12,0x1f000000);
	drawcircle((aX)*2,aY,32,16,side?0x7f9f1f1f:0x7f1f1f9f);
	static int inited = 0;
	if (!inited)
	{
		memcpy(gRVtx, gVtx, sizeof(vertex) * MAXVTX);
		inited = 1;
	}
		
	rotate_z(-rotz);
	rotate_y(gRoll);
	rotate_z(rotz);
	gRoll = 0;

	int i;
	for (i = 0; i < MAXVTX; i++)
	{
		if (gRVtx[i].z < 0)
		{
			drawcircle(aX*2+gRVtx[i].x * 30,
					   aY+gRVtx[i].y * 15,4,2,side?0xcfff7f7f:0xcf7f7fff);			
		}
	}
	for (i = 0; i < MAXVTX; i++)
	{
		if (gRVtx[i].z >= 0)
		{
			drawcircle(aX*2+gRVtx[i].x * 30,
					   aY+gRVtx[i].y * 15,4,2,0xcfffffff);			
		}
	}
}


static void draw_screen_ingame()
{
    int tick = SDL_GetTicks();

    int i, j;    

    if (tick - gLastTick > TICK_TIMEWARP) 
        gLastTick = tick;

    if (gLastTick >= tick)
    {
        SDL_Delay(1);
        return;
    }

    while (gLastTick < tick)
    {
		if (gState == STATE_INGAME)
			physics_tick(gLastTick);
		if (gState == STATE_PAUSE)
			pause_tick(gLastTick);


        gLastTick += 1000 / PHYSICS_FPS;
    }

    ////////////////////////////////////
    // Rendering
    ////////////////////////////////////

    glClearColor(0,0,0,1.0);
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	float camx = 200 - gCamx * 48;
	float camy = 225 - gCamy * 48;

	char *map = gMap[0];

	int xofs = (int)floor(playerx) - 7;
	int yofs = (int)floor(playery) - 7;
	if (xofs < 0) xofs = 0;
	if (yofs < 0) yofs = 0;

	int split;
	vidsplit(1);
	for (split = 0; split < 2; split++)
	{
		if (split)
		{
			//camx += 400;
			vidsplit(2);
			map = gMap[1];
		}

		// floor
		for (i = xofs; i < min(xofs+15,mapwidth); i++)
		{
			for (j = yofs; j < min(yofs+15,mapheight); j++)
			{
				int tileno = 0;
				switch (map[j*mapwidth+i])
				{
				case '#': tileno = 1; break;
				case ' ': tileno = 6; break;
				}
				if (tileno == 0)
					drawtile(tex_tile, i*48 + camx, j*48 + camy, 48, 48, tileno);
			}
		}

		for (i = 0; i < MAXCRUMBS; i++)
			if (gCrumb[i].live)
				drawtile(tex_tile, gCrumb[i].x*48 + camx, gCrumb[i].y*48 + camy, 48, 48, 3, (int)floor(0xaf * gCrumb[i].live / 1000.0) << 24);


		// overlay
		for (i = xofs; i < min(xofs+15,mapwidth); i++)
		{
			for (j = yofs; j < min(yofs+15,mapheight); j++)
			{
				int tileno = 0;
				switch (map[j*mapwidth+i])
				{
				case 'n': tileno = 2; break;
				case ',': tileno = 3; break;
				}
				if (tileno)
				{
					drawtile(tex_tile, i*48 + camx, j*48 + camy, 48, 48, tileno, 0xff000000);
					drawtile(tex_tile, i*48 + camx, j*48 + camy - bounce(tick + i * 10 + j * 100) * 4, 48, 48, tileno);
				}
			}
		}

		// player		
		drawplayer(playerx * 48 + camx, playery * 48 + camy, split);		

		// wall shadows
		for (i = xofs; i < min(xofs+15,mapwidth); i++)
		{
			for (j = yofs; j < min(yofs+15,mapheight); j++)
			{
				int tileno = 0;
				switch (map[j*mapwidth+i])
				{
				case '#': tileno = 1; break;
				}
				if (tileno == 1)
					drawrect((i*48 + camx - 24 + 12)*2, j*48 + camy - 24 + 12, 48*2, 48,0x2f000000);
			}
		}

		// walls
		for (i = xofs; i < min(xofs+15,mapwidth); i++)
		{
			for (j = yofs; j < min(yofs+15,mapheight); j++)
			{
				int tileno = 0;
				switch (map[j*mapwidth+i])
				{
				case '#': tileno = 1; break;
				}
				if (tileno == 1)
				{
					drawwall(tex_wall, i*48 + camx, j*48 + camy, 48, 48, ((i & 7) << 3) | (j & 7), 0xffffffff, gBevels[split][j*mapwidth+i]);
				}
			}
		}

		for (i = 0; i < MAXBLINKS; i++)
		{
			if (gBlink[i].live && gBlink[i].side & (1 << split))
			{
				drawrect((gBlink[i].x * 48 + camx - 24)*2, gBlink[i].y * 48 + camy - 24, 48*2, 48, 0xffffff | (int)floor(0xff * gBlink[i].live / BLINKAGE) << 24);
			}
		}

		drawsiderect(
			(destx*48 + camx - 24 + 12+(24 * (1 - bounce(tick)))/2)*2, 
			desty*48 + camy - 24  + 12+(24 * (1 - bounce(tick)))/2 - bounce(tick) * 4, 
			24*2 * bounce(tick), 
			24 * bounce(tick), 0x00ff00 | ((int)floor((bounce(tick)) * 0x3f) << 24));
	}
	vidsplit(0);

	int t = tick - gStateStartTick;

	if (gGameType == GAME_TIMED)
	{
		int tl = gTimeleft;
		if (tl <= 0) 
		{
			tl = 0;
			setstate(STATE_GAMEOVER);
			gStateStartTick = tick;
		}
		int s = tl / 1000;
		int m = s / 60;
		s %= 60;
		char temp[100];
		sprintf(temp, "%02d:%02d - Level %d", m, s, gLevel);
		quickfont_drawstring(tex_font, temp, 8, 8, 0x000000, 1, 4);
		quickfont_drawstring(tex_font, temp, 8, 8, 0x000000, 1, 4);
		quickfont_drawstring(tex_font, temp, 8, 8, 0xffffff, 1, 4);
	}

	if (t < 500)
	{
		float a = 1 - (t / 500.0f);
		drawrect(0,0,800,450,0xffffff | (int)floor(a * 0xff)<<24);
	}
    

	if (gState == STATE_PAUSE)
	{
		drawrect(0,0,800,450,0xa0808080);
		drawrect(0,190,800,50,0xa0000080);
		quickfont_drawstring(tex_font, "    Paused. Press esc to quit, fire to continue.", 0, 200, 0xffffff, 1, 2);
	}

#ifdef USE_PERFCOUNTERS

	render_perfcounters(tex_font);

#endif
    SDL_Delay(10);
	SDL_GL_SwapBuffers();
}


float menu_y_pos = 0;

static void draw_screen_menu()
{
    int tick = SDL_GetTicks();

    int i, j;    

    if (tick - gLastTick > TICK_TIMEWARP) 
        gLastTick = tick;

    if (gLastTick >= tick)
    {
        SDL_Delay(1);
        return;
    }

    while (gLastTick < tick)
    {
		if (gJoystick)
		{
			menu_y_pos += (SDL_JoystickGetAxis(gJoystick, 1) / 32768.0f) / 2;
		}
		if (gKeyState & KEY_UP)
		{
			menu_y_pos -= 0.5;
		}
		if (gKeyState & KEY_DOWN)
		{
			menu_y_pos += 0.5;
		}

		if (menu_y_pos < 0) menu_y_pos = 0;
		if (menu_y_pos >= 30) menu_y_pos = 29.9;

        gLastTick += 1000 / PHYSICS_FPS;
    }

	int pick = (int)floor(menu_y_pos / 10);

    ////////////////////////////////////
    // Rendering
    ////////////////////////////////////

    glClearColor(0,0,0,1.0);
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	quickfont_drawstring(tex_font, "Split//Maze", 8, 8, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 9, 8, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 8, 9, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 9, 9, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 10, 9, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 9, 10, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 10, 10, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 9, 9, 0x002000, 0.75, 4);
	drawrect(0,64,800,2,0xffffffff);
	quickfont_drawstring(tex_font, "By Jari Komppa for 30th Ludum Dare 48h game making contest - http://iki.fi/sol/", 76, 70, 0xffffff, 1, 1);
	quickfont_drawstring(tex_font, "By Jari Komppa for 30th Ludum Dare 48h game making contest - http://iki.fi/sol/", 76, 70, 0xffffff, 1, 1);

	quickfont_drawstring(tex_font, "Time Challenge", 200, 150, 0xffffff, (pick == 0)?1:0.75, 3);
	quickfont_drawstring(tex_font, "Endless Mode", 200, 200, 0xffffff, (pick == 1)?1:0.75, 3);
	quickfont_drawstring(tex_font, "Quit", 200, 250, 0xffffff, (pick == 2)?1:0.75, 3);

	quickfont_drawstring(tex_font, ">", 150, 140 + menu_y_pos * 4, 0xffffff, 0.75, 3);

	quickfont_drawstring(tex_font, "Use wasd, cursors, or gamepad / joystick. Controllers must be plugged in before starting the game.", 4, 430, 0xffffff, 0.75, 1);

	int fire = !!(gKeyState & KEY_FIRE);

	if (gJoystick)
	{
		int i;
		for (i = 0; i < 16; i++)
			if (SDL_JoystickGetButton(gJoystick, i))
				fire = 1;
	}

	if (fire)
	{
		switch (pick)
		{
		case 0:
			gGameType = GAME_TIMED;
			setstate(STATE_INGAME);
			gLevel = 1;
			gTimeleft = 10 * 1000;
			loadmap(gLevel);
			break;
		case 1:
			gGameType = GAME_ENDLESS;
			setstate(STATE_INGAME);
			gLevel = 1;
			loadmap(gLevel);
			break;
		case 2:
			exit(0);
		}
	}

#ifdef USE_PERFCOUNTERS

	render_perfcounters(tex_font);

#endif
    SDL_Delay(10);
	SDL_GL_SwapBuffers();
}

int wasfire = 0;

static void draw_screen_gameover()
{
    int tick = SDL_GetTicks();

    int i, j;    

    if (tick - gLastTick > TICK_TIMEWARP) 
        gLastTick = tick;

    if (gLastTick >= tick)
    {
        SDL_Delay(1);
        return;
    }

    while (gLastTick < tick)
    {
        gLastTick += 1000 / PHYSICS_FPS;
    }

	int pick = (int)floor(menu_y_pos / 10);

    ////////////////////////////////////
    // Rendering
    ////////////////////////////////////

    glClearColor(0,0,0,1.0);
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	int t = (tick - gStateStartTick) / 200;
	if (t > gLevel - 1) t = gLevel - 1;

	for (i = 0; i < t; i++)
	{
		drawrect(i * (800.0 / (gLevel-1)),
			     450 - timelefts[i] / 1000.0f,
				 800.0/(gLevel-1),
				 timelefts[i] / 1000.0f,
				 (i&1)?0xff40a040:0xff008000);
	}

	quickfont_drawstring(tex_font, "Split//Maze", 8, 8, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 9, 8, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 8, 9, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 9, 9, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 10, 9, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 9, 10, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 10, 10, 0xffffff, 1, 4);
	quickfont_drawstring(tex_font, "Split//Maze", 9, 9, 0x002000, 0.75, 4);
	drawrect(0,64,800,2,0xffffffff);
	quickfont_drawstring(tex_font, "By Jari Komppa for 30th Ludum Dare 48h game making contest - http://iki.fi/sol/", 76, 70, 0xffffff, 1, 1);
	quickfont_drawstring(tex_font, "By Jari Komppa for 30th Ludum Dare 48h game making contest - http://iki.fi/sol/", 76, 70, 0xffffff, 1, 1);



	quickfont_drawstring(tex_font, "Game over", 150, 150, 0xffffff, (pick == 0)?1:0.75, 3);
	char temp[100];
	sprintf(temp, "You got to level %d!", gLevel);
	quickfont_drawstring(tex_font, temp, 150, 200, 0xffffff, (pick == 0)?1:0.75, 3);
	quickfont_drawstring(tex_font, "Press fire to continue", 150, 300, 0xffffff, (pick == 0)?1:0.75, 3);


	int fire = !!(gKeyState & KEY_FIRE);

	if (gJoystick)
	{
		for (i = 0; i < 16; i++)
			if (SDL_JoystickGetButton(gJoystick, i))
				fire = 1;
	}

	if (!fire && wasfire)
	{
		setstate(STATE_MENU);
	}

	wasfire = fire;

#ifdef USE_PERFCOUNTERS

	render_perfcounters(tex_font);

#endif
    SDL_Delay(10);
	SDL_GL_SwapBuffers();
}

void draw_screen()
{
	switch (gState)
	{
	case STATE_PAUSE:
	case STATE_INGAME:
		draw_screen_ingame();
		break;
	case STATE_GAMEOVER:
		draw_screen_gameover();
		break;
	case STATE_MENU:
		draw_screen_menu();
		break;
	}
}


int main(int argc, char** args)
{	
    gVisualRand.init_genrand(0xc0cac01a);

    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE | SDL_INIT_JOYSTICK) < 0) 
    {
        fprintf(stderr, "Video initialization failed: %s\n", SDL_GetError());
        SDL_Quit();
        exit(0);
    }

    initvideo(argc);

	if (SDL_NumJoysticks() > 0)
	{
		gJoystick = SDL_JoystickOpen(0);		
		if (SDL_JoystickNumAxes(gJoystick) < 2)
		{
			// Not enough axes for our use; don't use the joystick.
			SDL_JoystickClose(gJoystick);
			gJoystick = NULL;
		}
	}
    tex_font = load_texture("data/fona.png");
    tex_tile = load_texture("data/tiles.png",1);
    tex_wall = load_texture("data/wall.png",0);

	gSoloud.init();
//	gSndCollided.loadPreset(SoLoud::Sfxr::EXPLOSION,1);
	gSndCollided.load("data/click.ogg");
	gSndCrumb.load("data/ding.ogg");	
	gSndNextLevel.load("data/tadah.ogg");

	gMusicFilter.setParams(SoLoud::BiquadResonantFilter::HIGHPASS,44100,4000,1);
	gMusicBus.setFilter(0, &gMusicFilter);
	musichandle = gSoloud.play(gMusicBus);
	gSoloud.setVolume(musichandle, 2);
	gSoloud.setFilterParameter(musichandle,0,0,0);



	gSndMusicTimed.load("data/timeattack2.ogg");
	gSndMusicTimed.setLooping(1);
	musichandletimed = gMusicBus.play(gSndMusicTimed,0,0,1);
	gSoloud.setProtectVoice(musichandletimed, 1);

	gSndMusicEndless.load("data/endless2.ogg");
	gSndMusicEndless.setLooping(1);
	musichandleendless = gMusicBus.play(gSndMusicEndless,0,0,1);
	gSoloud.setProtectVoice(musichandleendless, 1);

	gSndMusicMenu.load("data/menu2.ogg");
	gSndMusicMenu.setLooping(1);
	musichandlemenu = gMusicBus.play(gSndMusicMenu);
	gSoloud.setProtectVoice(musichandlemenu, 1);

	fxhandle = gSoloud.play(gSndFxBus);
	gSoloud.setVolume(fxhandle, 0.5);
	
//	gSndCrumb.loadPreset(SoLoud::Sfxr::COIN,1);
//	gSndNextLevel.loadPreset(SoLoud::Sfxr::JUMP,1);
	//gSoloud.setGlobalVolume(4);
	
	gVtx = new vertex[MAXVTX];
	gRVtx = new vertex[MAXVTX];

	int i;
	for (i = 0; i < MAXVTX / 3; i++)
	{
		gVtx[i].x = sin(6 * M_PI * i / (float)MAXVTX);//(gPhysicsRand.genrand_int31() % 32768) - 16384.0f;
		gVtx[i].y = cos(6 * M_PI * i / (float)MAXVTX);//(gPhysicsRand.genrand_int31() % 32768) - 16384.0f;
		gVtx[i].z = 0;//(gPhysicsRand.genrand_int31() % 32768) - 16384.0f;
		float len = (float)sqrt(gVtx[i].x * gVtx[i].x + 
								gVtx[i].y * gVtx[i].y + 
								gVtx[i].z * gVtx[i].z);
		if (len != 0)
		{
			gVtx[i].x /= len;
			gVtx[i].y /= len;
			gVtx[i].z /= len;
		}
	}

	for (; i < 2 * MAXVTX / 3; i++)
	{
		gVtx[i].x = sin(6 * M_PI * i / (float)MAXVTX);//(gPhysicsRand.genrand_int31() % 32768) - 16384.0f;
		gVtx[i].z = cos(6 * M_PI * i / (float)MAXVTX);//(gPhysicsRand.genrand_int31() % 32768) - 16384.0f;
		gVtx[i].y = 0;//(gPhysicsRand.genrand_int31() % 32768) - 16384.0f;
		float len = (float)sqrt(gVtx[i].x * gVtx[i].x + 
								gVtx[i].y * gVtx[i].y + 
								gVtx[i].z * gVtx[i].z);
		if (len != 0)
		{
			gVtx[i].x /= len;
			gVtx[i].y /= len;
			gVtx[i].z /= len;
		}
	}

	for (; i < MAXVTX; i++)
	{
		gVtx[i].y = sin(6 * M_PI * i / (float)MAXVTX);//(gPhysicsRand.genrand_int31() % 32768) - 16384.0f;
		gVtx[i].z = cos(6 * M_PI * i / (float)MAXVTX);//(gPhysicsRand.genrand_int31() % 32768) - 16384.0f;
		gVtx[i].x = 0;//(gPhysicsRand.genrand_int31() % 32768) - 16384.0f;
		float len = (float)sqrt(gVtx[i].x * gVtx[i].x + 
								gVtx[i].y * gVtx[i].y + 
								gVtx[i].z * gVtx[i].z);
		if (len != 0)
		{
			gVtx[i].x /= len;
			gVtx[i].y /= len;
			gVtx[i].z /= len;
		}
	}


	// For imgui - Enable keyboard repeat to make sliders more tolerable
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
	// For imgui - Enable keyboard UNICODE processing for the text field.
	SDL_EnableUNICODE(1);

    // load a font..
//    fn.load("vera20.fnt");
    
    // set window title
    SDL_WM_SetCaption(TITLE " - http://iki.fi/sol/", NULL);

    // hide cursor
    SDL_ShowCursor(0);
/*
	for (i = 0; i < 100; i++)
	{
		loadmap(i);
		char temp[100];
		sprintf(temp, "\n%d\t%d", gPathLen, i);
		OutputDebugStringA(temp);
	}
*/

//	SDL_SetCursor(load_cursor("cursor_scissors.png",0,0));
    
    while (1) 
    {
        process_events();
        draw_screen();
    }

    return 0;
}
